package Sman; 

#$Id: Sman.pm,v 1.27 2004/06/06 17:08:54 joshr Exp $

require 5.006;
use strict;
use warnings;
our $VERSION = '0.9.1';
our $SMAN_DATA_VERSION = "1.0"; 	# this is only relevant to Sman

1;
__END__

=head1 NAME

Sman - Perl tool for searching man pages

=head1 SYNOPSIS

See 'sman --help'

=head1 DESCRIPTION 

Sman is a Searcher for Man pages. 

=head1 AUTHOR

Josh Rabinowitz

=head1 SEE ALSO

L<sman>, L<sman-update>, L<sman.conf>

=cut
